/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem

  -----------------------------------------------
  Umwandlung users.bcm/users3.bcm nach users4.bcm
  -----------------------------------------------

  Copyright (C)       Markus Baumann
                      Am Altenfelder Kreuz 43
                      59302 Oelde

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <mem.h>
#include <string.h>
#include <conio.h>

#undef fopen
#undef fclose
#define USERNAMEDOS      "users.bcm"
#define USERNAMEW32LINUX "users3.bcm"
#define USERNAMENEW      "users4.bcm"

#define CALLEN          6
#define NAMELEN         14
#define QTHLEN          30
#define ZIPLEN          8
#define MYBBSLEN        39     // length of complete MyBBS incl. H-Route
#define CALLSSID        9      // call with SSID
#define PROMPTLEN       20
#define LINELEN         80     // standard line length
#define FIRSTCMDLEN     20
#define OLDMYBBSLEN     24     // for compatibility with the old mybbs field

typedef unsigned long bitfeld;

typedef struct user_t
//*************************************************************************
//
//  Struktur fuer die users4.bcm Userdatenbank
//
//*************************************************************************
{ char     call[CALLEN+1];          //  7 Rufzeichen des Users
  char     name[NAMELEN+1];         // 15 A NAME
  char     qth[QTHLEN];             // 30 QTH (DH8YMB)
  char     qthok;                   //  1 QTH selbst eingegeben
  char     zip[ZIPLEN];             //  8 ZIP-Code (Postleitzahl) (DH8YMB)
  char     zipok;                   //  1 Zip-Code selbst eingegeben
  char     mybbs[MYBBSLEN+1];       // 40 A FORWARD
  char     mybbsok;                 //  1 MYBBS selbst eingegeben
  time_t   mybbstime;               //  4 Datum des letzten MYBBS (UNIX)
  char     uplink[CALLSSID+1];      // 10 uplink-Digi
  char     newcall[CALLEN+1];       //  7 New callsign (license update)
  char     prompt[PROMPTLEN+1];     // 21 A PROMPT
  char     sprache[4];              //  4 A SPEECH
  char     password[40];            // 40 A PW (pw off = 0x01 0x00 ...)
  char     notvisible[LINELEN];     // 80 A REJECT
  char     firstcmd[FIRSTCMDLEN+1]; // 21 A COMMAND
  short unsigned helplevel;         //  2 A HELPLEVEL
  long unsigned logins;             //  4 Anzahl an Logins
  short unsigned lf;                //  2 A LF
  short unsigned zeilen;            //  2 A LINES
  char     nopurge;                 //  1 Board wird nicht geputzt
  char     nameok;                  //  1 Name selbst eingegeben
  time_t   lastboxlogin;            //  4 letzter Login (UNIX)
  time_t   lastdirnews;             //  4 letztes DIR NEWS / Check (UNIX)
  time_t   lastquit;                //  4 letztes QUIT
  time_t   lastload;                //  4 interner Merker fr Verwaltung
  long     mailsent;                //  4 gesendete Nachrichten
  long     mailgot;                 //  4 erhaltene Nachrichten
  long     mailread;                //  4 gelesene Nachrichten
  long unsigned  daybytes;          //  4 gelesene Bytes heute
  bitfeld  opt[8];                  // 32 eingestellte Optionen
  char     ufwd[40];                // 40 Connect-Path fuer User-forwarding
  char     loginpwtype;             //  1 Password-type for ax-login (DH3MB)
                                    //    0..BayCom-Password
                                    //    1..MD2
                                    //    2..MD5
                                    //    100..disabled
  char     sfpwtype;                //  1 Password-type for S&F (DH3MB)
                                    //    0..BayCom-Password
                                    //    1..MD2
                                    //    2..MD5
                                    //    100..disabled
  char     ttypw[9];                //  9 A TTYPW (TTY-Password)
  short unsigned fdelay;            //  2 Verzoegerung fr eigene Nachrichten
  short unsigned fhold;             //  2 Verzoegerung fr Nachrichten
  short unsigned status;            //  2 A STATUS (Bit-field)
                                    //    0..default
                                    //    1..no quota-limits
                                    //    2..access denied
                                    //    4..access denied, sysop informed
                                    //    8..my not use transfer command
  char     readlock;                //  1 Privatmails nicht lesbar
  char     rlimit;                  //  1 fremde Privatmails nicht lesbar
  char     pwline;                  //  1 Leerzeile beim Login
  char     echo;                    //  1 Eigene Eingaben zurckschicken
  short unsigned paclen;            //  2 Packetlength
  char     dirformat;               //  1 Dir/Check/List in new format
  char     binmode;                 //  1 AutoBIN/YAPP/DIDADIT bei BIN-Mails
  char     unsecure_smtp;           //  1  0=>needs to pop before smtp
                                    //     1=>smtp without pop allowed
  char     away;                    //  1 AWAY-Funktion aktiv/deaktiv (DH8YMB)
  time_t   awayendtime;             //  4 AWAY Endtime (UNIX)
  char     awaytext[LINELEN];       // 80 AWAY Text
  char     charset;                 //  1 A CHARSET (nicht implementiert)
  char     ttycharset;              //  1 TTY-Charset (nicht implementiert)

  char     restplatz[109];          //109 Restplatz

  short unsigned filepos_old;       //  2 Position im Userfile (altes Format)
  short unsigned nextsamehash_old;  //  2 Link Hashliste (altes Format)
  long unsigned filepos;            //  4 Position im Userfile
  long unsigned nextsamehash;       //  4 Link Hashliste
                                    //  = 640 Bytes
} user_t ;

/*---------------------------------------------------------------------------*/

typedef struct userdos_t
//*************************************************************************
//
//  Struktur fuer die alten Userdaten users.bcm (DOS)
//
//*************************************************************************
{ char     call[CALLEN+1];          //  7 Rufzeichen des Users
  char     name[NAMELEN+1];         // 15 A NAME
  char     oldmybbs[OLDMYBBSLEN+1]; // 25 for compatibility with old mybbs field
  char     prompt[PROMPTLEN+1];     // 21 A PROMPT
  char     sprache[4];              //  4 A SPEECH
  char     password[40];            // 40 A PW ; pw off = 0x01 0x00 ..
  char     notvisible[LINELEN];     // 80 A REJECT
  char     firstcmd[FIRSTCMDLEN+1]; // 21 A COMMAND
  short unsigned helplevel;         //  2 A HELPLEVEL
  short unsigned zeilen;            //  2 A LINES
  char     nopurge;                 //  1 Board wird nicht geputzt
  char     nameok;                  //  1 Name selbst eingegeben
  time_t   lastboxlogin;            //  4 letzter Login (UNIX)
  time_t   lastdirnews;             //  4 letztes DIR NEWS / Check (UNIX)
  time_t   lastquit;                //  4 letztes QUIT
  long     mailsent;                //  4 gesendete Nachrichten
  long     mailgot;                 //  4 erhaltene Nachrichten
  long     mailread;                //  4 gelesene Nachrichten
  long unsigned logins;             //  4 logins
  char     mybbsok;                 //  1 MYBBS selbst eingegeben (1)
  bitfeld  opt[8];                  // 32 eingestellte Optionen
  short unsigned lf;                //  2 A LF
  char     charset;                 //  1 A CHARSET (nicht impl)
  time_t   mybbstime;               //  4 Datum des letzten MYBBS (UNIX)
  char     ttypw[9];                //  9 A TTYPW (TTY-Password)
  time_t   lastload;                //  4 interner Merker fr Verwaltung
  long unsigned  daybytes;          //  4 gelesene Bytes heute
  short unsigned status;            //  2 A STATUS (Bit-field)
                                    //    0..default
                                    //    1..no quota-limits
                                    //    2..access denied
                                    //    4..access denied, sysop informed
                                    //    8..my not use transfer command
  char     uplink[CALLSSID+1];      // 10 uplink-Digi
  char     readlock;                //  1 Privatmails nicht lesbar (")
  char     pwline;                  //  1 Leerzeile beim Login
  char     echo;                    //  1 Eigene Eingaben zurckschicken
  short unsigned fdelay;            //  2 Verzoegerung fr eigene Nachrichten
  short unsigned fhold;             //  2 Verzoegerung fr Nachrichten
  char     rlimit;                  //  1 fremde Privatmails nicht lesbar
  char     ufwd[40];                // 40 Connect-Path fuer User-forwarding
  char     loginpwtype;             //  1 Password-type for ax-login (DH3MB)
                                    //    0..BayCom-Password
                                    //    1..MD2
                                    //    2..MD5
                                    //    100..disabled
  char     sfpwtype;                //  1 Password-type for S&F (DH3MB)
                                    //    0..BayCom-Password
                                    //    1..MD2
                                    //    2..MD5
                                    //    100..disabled
  char newcall[CALLEN+1];           //  7 New callsign (license update)
  short unsigned paclen;            //  2 Packetlength
  char dirformat;                   //  1 Dir/Check/List in new format
  char binmode;                     //  1 AutoBIN/YAPP/DIDADIT bei BIN-Mails
  char unsecure_smtp;               //  1  0=>needs to pop before smtp
                                    //     1=>smtp without pop allowed
  char mybbs[MYBBSLEN+1];           // 40 A FORWARD
  char away;                        //  1 AWAY-Funktion aktiv (DH8YMB)
  char restplatz[81];               // 81 ist noch frei (definiert 0)
  long unsigned filepos;            //  4 Position im Userfile
  long unsigned nextsamehash;       //  4 hash list link
  short unsigned filepos_old;       //  2 (obsolete) Position im Userfile
  short unsigned nextsamehash_old;  //  2 (obsolete) hash list link
                                    //  = 512 Bytes
} userdos_t ;

/*---------------------------------------------------------------------------*/

typedef struct userw32linux_t
//*************************************************************************
//
//  Struktur fuer die alten Userdaten users3.bcm der WinNT/Linux-Version,
//  beruecksichtigt Alignment vom GCC (mind. 4 Byte)
//
//*************************************************************************
{ time_t   lastboxlogin;                //  4 letzter Login (UNIX)
  time_t   lastdirnews;                 //  4 letztes DIR NEWS / Check (UNIX)
  time_t   lastquit;                    //  4 letztes QUIT
  time_t   mybbstime;                   //  4 Datum des letzten MYBBS (UNIX)
  time_t   lastload;                    //  4 interner Merker fr Verwaltung
  long unsigned daybytes;               //  4 gelesene Bytes heute
  long     mailsent;                    //  4 gesendete Nachrichten
  long     mailgot;                     //  4 erhaltene Nachrichten
  long     mailread;                    //  4 gelesene Nachrichten
  long unsigned logins;                 //  4 logins
  bitfeld  opt[8];                      // 32 eingestellte Optionen
  long     spare_long[10];
  short unsigned helplevel;             //  2 A HELPLEVEL
  short unsigned zeilen;                //  2 A LINES
  short unsigned lf;                    //  2 A LF
  short unsigned status;                //  2 A STATUS (Bit-Field)
  short unsigned filepos_old;           //  2 (obsolete)Position im Userfile
  short unsigned nextsamehash_old;      //  2 (obsolete)hash list link
  short unsigned fdelay;                //  2 Verzoegerung fr eigene Nachrichten
  short unsigned fhold;                 //  2 Verzoegerung fr Nachrichten
  short unsigned paclen;                //  2 packet length
  short unsigned spare_short[19];
  char     call[CALLEN+1];              //  7 Rufzeichen des Users
  char     name[NAMELEN+1];             // 15 A NAME
  char     oldmybbs[OLDMYBBSLEN+1];     // 25 for compatibility with old mybbs field
  char     prompt[PROMPTLEN+1];         // 21 A PROMPT
  char     sprache[4];                  //  4 A SPEECH
  char     password[40];                // 40 A PW
  char     notvisible[LINELEN];         // 80 A REJECT
  char     firstcmd[FIRSTCMDLEN+1];     // 21 A COMMAND
  char     nopurge;                     //  1 Board wird nicht geputzt
  char     nameok;                      //  1 Name selbst eingegeben
  char     mybbsok;                     //  1 MYBBS selbst eingegeben (1)
  char     charset;                     //  1 A CHARSET (nicht impl)
  char     ttypw[9];                    //  9 A TTYPW (TTY-Password)
  char     uplink[CALLSSID+1];          // 10 uplink-Digi
  char     readlock;                    //  1 Privatmails "nicht lesbar"
  char     pwline;                      //  1 Leerzeile beim Login
  char     echo;                        //  1 Eigene Eingaben zurckschicken
  char     rlimit;
  char     ufwd[40];                    // 40 Connect-Path fuer User-forwarding
  char     loginpwtype;                 //  1 Password-type for ax-login (DH3MB)
  char     sfpwtype;                    //  1 Password-type for S&F (DH3MB)
  char     newcall[CALLEN+1];           //  7 New callsign (lizence update)
  char     dirformat;                   //  1 Dir/Check/List in new format
  char     unsecure_smtp;               //  1  0=>needs to pop before smtp
                                        //     1=>smtp without pop allowed
  char     binmode;                     //  1 AutoBIN/YAPP/DIDADIT bei BIN-Mails
  char     ttycharset;                  //  1 A TTYCHARSET (nicht impl)
  char     mybbs[MYBBSLEN+1];           // 40 A FORWARD
  char     away;                        //  1 AWAY-Funktion aktiv (DH8YMB)
  char     restplatz[2];                //  2 ist noch frei (definiert 0)
  unsigned long filepos;                //  4 position within file
  unsigned long nextsamehash;           //  4 hash to next record
                                        // = 512 Bytes (473)
} userw32linux_t ;

/*---------------------------------------------------------------------------*/

void main(void)
//*************************************************************************
//
//  Konvertiert users.bcm/users3.bcm nach users4.bcm
//  Dauer bei Pentium1 mit 133MHz und ca. 16.800 Usern: ca. 4 Sekunden
//
//*************************************************************************
{   FILE *fo;
    FILE *fn;
    int i,auswahl;
    long anzahl=0;
    auswahl=0;
    user_t us;
    userdos_t uso1;
    userw32linux_t uso2;

    printf("CVUSERS4 - Userdatabase convert util for OpenBCM\n");
    printf("10/2003 by Markus Baumann, DH8YMB\n\n");
    printf("This program converts users.bcm/users3.bcm -> users4.bcm\n\n");
    printf("Which file do you want to convert?\n");
    printf("<1> users.bcm  -> users4.bcm\n");
    printf("<2> users3.bcm -> users4.bcm\n");
    while (! ((auswahl==49) || (auswahl==50)))
    // Tastaturcode: 49 = 1 , 50 = 2
    {  printf("Your selection: ");
       auswahl=getch();
       printf("\n");
    }
    printf("\nTry processing userdata...\n");
    if (auswahl==49)
    {
      fo = fopen (USERNAMEDOS, "rb");
      if(!fo)
      { printf("Can't open %s\n",USERNAMEDOS);
        exit(1);
      }
    }
    if (auswahl==50)
    {
      fo = fopen (USERNAMEW32LINUX, "rb");
      if(!fo)
      { printf("Can't open %s\n",USERNAMEW32LINUX);
        exit(1);
      }
    }
    fn = fopen (USERNAMENEW, "wb");
    if(!fn)
    { printf("Can't open %s\n",USERNAMENEW);
        exit(1);
    }
    setvbuf(fo,NULL,_IOFBF,16384);
    setvbuf(fn,NULL,_IOFBF,16384);
    if (fo && fn)
    {
      printf("Now start converting...\n");
      anzahl=0;
      if (auswahl==49)
      while(fread(&uso1,sizeof(userdos_t), 1, fo)==1)
      { memset(&us,0,sizeof(user_t));
        us.lastboxlogin=uso1.lastboxlogin;
        us.lastdirnews=uso1.lastdirnews;
        us.lastquit=uso1.lastquit;
        us.mybbstime=uso1.mybbstime;
        us.lastload=uso1.lastload;
        us.daybytes=uso1.daybytes;
        us.mailsent=uso1.mailsent;
        us.mailgot=uso1.mailgot;
        us.mailread=uso1.mailread;
        us.logins=uso1.logins;
        for(i=0; i<8; i++)
          us.opt[i]=uso1.opt[i];
        us.helplevel=uso1.helplevel;
        us.zeilen=uso1.zeilen;
        us.lf=uso1.lf;
        us.status=uso1.status;
        us.filepos=uso1.filepos;
        us.nextsamehash=uso1.nextsamehash;
        us.fdelay=uso1.fdelay;
        us.fhold=uso1.fhold;
        us.paclen=uso1.paclen;
        us.readlock=uso1.readlock;
        us.pwline=uso1.pwline;
        us.echo=uso1.echo;
        us.rlimit=uso1.rlimit;
        us.nopurge=uso1.nopurge;
        us.nameok=uso1.nameok;
        us.mybbsok=uso1.mybbsok;
        us.charset=uso1.charset;
        us.loginpwtype=uso1.loginpwtype;
        us.sfpwtype=uso1.sfpwtype;
        us.dirformat=uso1.dirformat;
        us.unsecure_smtp=uso1.unsecure_smtp;
        us.binmode=uso1.binmode;
        us.away=uso1.away;
        us.awayendtime=0;
        *us.awaytext = 0;
        strcpy(us.qth,"?");
        us.qthok=0;
        strcpy(us.zip,"?");
        us.zipok=0;
        *us.restplatz = 0;
        strcpy(us.call,uso1.call);
        strcpy(us.name,uso1.name);
        if (*uso1.mybbs) strcpy(us.mybbs,uso1.mybbs);
        else strcpy(us.mybbs,uso1.oldmybbs);
        strcpy(us.prompt,uso1.prompt);
        strcpy(us.sprache,uso1.sprache);
        strcpy(us.password,uso1.password);
        strcpy(us.notvisible,uso1.notvisible);
        strcpy(us.firstcmd,uso1.firstcmd);
        strcpy(us.ttypw,uso1.ttypw);
        strcpy(us.uplink,uso1.uplink);
        strcpy(us.ufwd,uso1.ufwd);
        strcpy(us.newcall,uso1.newcall);
        fwrite(&us, sizeof(user_t), 1, fn);
        anzahl++;
      }
      if (auswahl==50)
      while(fread(&uso2,sizeof(userw32linux_t), 1, fo)==1)
      {
        memset(&us,0,sizeof(user_t));
        us.lastboxlogin=uso2.lastboxlogin;
        us.lastdirnews=uso2.lastdirnews;
        us.lastquit=uso2.lastquit;
        us.mybbstime=uso2.mybbstime;
        us.lastload=uso2.lastload;
        us.daybytes=uso2.daybytes;
        us.mailsent=uso2.mailsent;
        us.mailgot=uso2.mailgot;
        us.mailread=uso2.mailread;
        us.logins=uso2.logins;
        for(i=0; i<8; i++)
          us.opt[i]=uso2.opt[i];
        us.helplevel=uso2.helplevel;
        us.zeilen=uso2.zeilen;
        us.lf=uso2.lf;
        us.status=uso2.status;
        us.filepos=uso2.filepos;
        us.nextsamehash=uso2.nextsamehash;
        us.fdelay=uso2.fdelay;
        us.fhold=uso2.fhold;
        us.paclen=uso2.paclen;
        us.readlock=uso2.readlock;
        us.pwline=uso2.pwline;
        us.echo=uso2.echo;
        us.rlimit=uso2.rlimit;
        us.nopurge=uso2.nopurge;
        us.nameok=uso2.nameok;
        us.mybbsok=uso2.mybbsok;
        us.charset=uso2.charset;
        us.loginpwtype=uso2.loginpwtype;
        us.sfpwtype=uso2.sfpwtype;
        us.dirformat=uso2.dirformat;
        us.unsecure_smtp=uso2.unsecure_smtp;
        us.binmode=uso2.binmode;
        us.ttycharset=uso2.ttycharset;
        us.away=uso2.away;
        us.awayendtime=0;
        *us.awaytext = 0;
        strcpy(us.qth,"?");
        us.qthok=0;
        strcpy(us.zip,"?");
        us.zipok=0;
        *us.restplatz = 0;
        strcpy(us.call,uso2.call);
        strcpy(us.name,uso2.name);
        if (*uso2.mybbs) strcpy(us.mybbs,uso2.mybbs);
        else strcpy(us.mybbs,uso2.oldmybbs);
        strcpy(us.prompt,uso2.prompt);
        strcpy(us.sprache,uso2.sprache);
        strcpy(us.password,uso2.password);
        strcpy(us.notvisible,uso2.notvisible);
        strcpy(us.firstcmd,uso2.firstcmd);
        strcpy(us.ttypw,uso2.ttypw);
        strcpy(us.uplink,uso2.uplink);
        strcpy(us.ufwd,uso2.ufwd);
        strcpy(us.newcall,uso2.newcall);
        fwrite(&us, sizeof(user_t), 1, fn);
        anzahl++;
      }
    printf("%ld users converted.\n",anzahl);
    }
    if (fo) fclose (fo);
    if (fn) fclose (fn);
    printf("cvusers4 has finished.\n");
}
